# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: TonieToolbox/tonie_header.proto
# Protobuf Python Version: 6.32.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    32,
    1,
    '',
    'TonieToolbox/tonie_header.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fTonieToolbox/tonie_header.proto\x12\x05tonie\"q\n\x0bTonieHeader\x12\x10\n\x08\x64\x61taHash\x18\x01 \x01(\x0c\x12\x12\n\ndataLength\x18\x02 \x01(\r\x12\x11\n\ttimestamp\x18\x03 \x01(\r\x12\x18\n\x0c\x63hapterPages\x18\x04 \x03(\rB\x02\x10\x01\x12\x0f\n\x07padding\x18\x05 \x01(\x0c\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'TonieToolbox.tonie_header_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_TONIEHEADER'].fields_by_name['chapterPages']._loaded_options = None
  _globals['_TONIEHEADER'].fields_by_name['chapterPages']._serialized_options = b'\020\001'
  _globals['_TONIEHEADER']._serialized_start=42
  _globals['_TONIEHEADER']._serialized_end=155
# @@protoc_insertion_point(module_scope)
