#!/usr/bin/env python3
"""
Minimal setup.py for backward compatibility.
All configuration is now in pyproject.toml.
"""
from setuptools import setup

# All package configuration is now defined in pyproject.toml
# This setup.py exists only for backward compatibility with older tools
setup()
